# Bases de datos elegidas
## MongoDB
Probablemente la más conocida, ha conseguido convertirse en una de las startups más conocidas. Esta NoSQL es usada por Foursquare y eBay.

Las ventajas que MongoDB ofrece frente a las bases de datos relacionales como PostgreSQL o MySQL y por las cuales muchas compañías optan por el uso de MongoDB suele ser la capacidad de crear aplicaciones más rápidamente, manejar tipos de datos diversos y tener una mayor escalabilidad, se administra eficientemente de manera más fácil.

Las aplicaciones se crean más rápidamente porque los documentos de MongoDB se corresponden de forma natural con los lenguajes orientados a objetos de hoy en dia. MongoDB elimina la capa de mapeo objeto relacional complejo (ORM), el cual traduce objetos en el código a tablas relacionales.

También, el modelo de datos flexible que proporciona MongoDB hace que el esquema de la base de datos evolucione en armonía con los requisitos del negocio. De esta forma, cambios en el esquema pueden tardar días en un modelo relacional, mientras que con MongoDB solo horas.

Esto hace que MongoDB sea escalable entre Centros de Datos Distribuidos, dotando las infraestructuras de mayor nivel de disponibilidad y escalabilidad, que con un modelo relacional no se puede conseguir. Sin tiempo de inactividad y sin cambiar la aplicación se puede escalar fácilmente a medida que la implementación crece en función de volumen de datos y rendimientos. Con un modelo relacional como MySQL, la escalabilidad requiere trabajo adicional, costoso. La empresa china Baidu, migró de MySQL a MongoDB para apoyar el rápido crecimiento de negocio y ahora tiene más de 100 aplicaciones administrando más de 1PB (10^15 bytes) de datos con un cluster de MongoDB [1].

Por otro lado las desventajas de MongoDB van relacionadas al uso de transacciones multi columnas completas, cuando se necesita persistencia en los datos o el modelo es muy complejo. Un ejemplo concreto es un motor que tienen los sistemas de reserva de viajes, los cuales siempre suelen tener transacciones complejas. 

No es raro tener un sistema híbrido con MySQL y MongoDB. Siguiendo el ejemplo anterior, se podría dejar el núcleo de las transacciones para MySQL y las partes del sistema que incluya usuarios, gestión de sesiones, prestación de contenido, integración con redes sociales, etc. delegarlas en MongoDB.

El servicio online de MongoDB, o database as service está disponible por mLab [2], el cual da servicio a empresas como FOX o Toyota.

## CouchDB
Es la base de datos orientada a documentos de Apache. La principal característica y por la que se suele escoger es que todos sus datos son accesibles a través de una API Rest. Esta NoSQL es utilizada por la BBC y Credit Suisse.

La principal característica que tiene aparte de la semántica ACID con un controlador de concurrencia es la interfaz rest que ninguna relacional proporciona automáticamente. En CouchDB todos los elementos tienen una URI expuesta por HTTP. De esta forma, el servicio REST aplica las operaciones CRUD a los métodos HTTP POST, GET, PUT y DELETE.

Además, al ser del mismo tipo que MongoDB y almacena los datos como documentos (en este caso en JSON).  Pueden ser fechas, números, caracteres, fechas, listas y vectores. Todos los documentos tienen su propio identificador como hemos dicho antes y no necesitan un esquema determinado, con las ventajas y desventajas que ello implica (visto antes) [3].

CouchDB está diseñado para almacenar y reportar grandes cantidades de datos semi estructurado, a diferencia de una base de datos relacional. En una relacional, el esquema definido provoca que la información sea limitada, sobretodo cuando se da lugar el caso en el que las necesidades cambian respecto a las del diseño, provocando problemas de actualización en un medio distribuido. Con CouchDB, al no estar forzado por el esquema  lo nuevos tipos de datos pueden ser añadidos sin problemas.

Para gestionar los documentos, CouchDB integra un modelo de vistas que utilizan JavaScript, las cuales permiten agregar y reportar en los documentos en la base de datos. Las vistas son construidas dinámicamente y no afectan por debajo al documento, pudiendo tener tantas vistas diferentes de los mismos datos como se desee.

Además está preparada para trabajar offline; replicar los datos y actualizar automáticamente cuando se vuelva a tener conexión. Todo ello controlado a través de líneas de comandos o su interfaz de administración incorporada y accesible por web: Futon [5].

## Cassandra
Es un modelo híbrido derivado de querer obtener las ventajas de las bases de datos orientadas a columnas (BigTable) y de clave-valor (Dynamo) desarrollado en Java. Esta base de datos es usada por Facebook, Twitter (dejaron de usarla para almacenar tweets), Netflix (back-end del streaming).

Cassandra es de código abierto, la principal característica es que fusiona Dynamo (de Amazon) con BigTable (de Google), siendo estas dos de código cerrado. Cassandra nació de la necesidad de Facebook en resolver problemáticas derivadas del motor de búsqueda entre usuarios.

Como hemos mencionado, esta base de datos combina la consistencia eventual de Dynamo con las familias de columnas de BigTable siendo gratuita y libre.

Como resultado el modelo de Cassandra típico es una tabla por instancia y cada familia de columnas puede contener más columnas o agrupaciones de estas. La columna es la que que contiene elementos tipo clave-valor-tiempo, este último valor definible por el usuario. Al final, cada fila de una tabla puede tomar valores de las columnas distintas de una familia de ellas que otra fila (si tenemos 4 columnas en una familia: 1,2,3,4 la fila F1 puede tener valores en 1 y 2 y F2 en 1,3,4) [7].

Cassandra recibe las características implícitas de los modelos NoSQL [4] con los mencionados anteriormente: 
* Es descentralizada: no todos los nodos tienen un mismo rol.
* Escalable: esquema no definido, el rendimiento de escritura/lectura aumenta en función de los nodos añadidos.
* Permite consistencia: se ofrece un nivel de consistencia para lecturas y escrituras.
* Tolerante a fallos: los datos se replican en nodos.
* Replicación: cassandra está diseñado como un sistema distribuido y las estrategias de replicación son configurables según la necesidad del usuario.
* MapReduce: Al estar dentro del proyecto Apache Hadoop soporta sus tecnologías, inclusive Pig y Hive.
* Lenguaje de consulta: Utiliza CQL (Cassandra Query Language), una alternativa a SQL.

Cassandra (Apache), no tiene servicio propio online. Solo a partir de las versiones 1.2.x [6] que utilizan CQL3 se puede configurar data service. El DBASS (database as service) lo proporciona Datastax Academy [8] cuyo producto proporciona la administración de los nodos de Cassandra y cuadros de mando que representan el estado y funcionamiento de estos mismo. El producto tiene una versión libre (community) bastante completa (previo registro) y bastante características de análisis y gráficas características de los cuadros de mando.

## HBase
Desarrollada en Java y mantenida por Hadoop Apache, se utiliza para procesar grandes cantidades de datos y la alternativa a BigTable. Es común en los sistemas de Big Data los cuales procesan y almacenan grandes cantidades de información. Las empresas que la usan son Facebook para la plataforma de mensajería, Airbnb como parte de su sistema de cálculo de tiempo real o Spotify para machine learning.

HBase es una NoSQL y por tanto hereda las características propias como tal, ya expuestas y explicadas previamente. Las características principales de HBase son: es de baja latencia, aprovecha el sistema de archivos distribuidos de Hadoop y usa el HDFS (sistema de ficheros distribuido, escalable y portátil escrito en Java y creado especialmente para trabajar con ficheros de gran tamaño) de Hadoop como capa de persistencia.

Esta base de datos, como ya se ha mencionado anteriormente, almacena la información como tuplas clave-valor y el esquema de la tabla es flexible, dividiendo la información en familias de columnas, haciéndolo totalmente flexible, ya que las columnas de cada una de las familias pueden definirse a nivel de fila; cosa completamente impensable en las bases de datos relacionales [13].

Además, toda la información almacenada se mantiene controlada mediante versiones de forma transparente, dando la posibilidad de acceso a todos lo valores que ha tenido una columna de una fila mediante el timestamp del momento en el que se cambio el momento del valor.
Hbase realiza las consultas mediante un lenguaje parecido a javascript, cosa que no puede ser intuitiva. No obstante HBase nos puede manejar y recuperar información del orden de 10^8 tuplas o superior, medida con la cual las bases de datos relacionales ya empiezan mostrar tiempos excesivos.

Aunque HBase no tiene un database as service específico, si que se suele manejar de forma online con Ambari (también de Apache) [9], el cual gestiona el entorno de Hadoop y entre ellos HBase.

## DynamoDB
Es la base de datos desarrollada por amazon y es la opción que almacén que podemos utilizar a través de su AWS. Es utilizada por Amazon, Washintong Post y Scopely.

DynamoBD es un servicio de base de datos en la nube mediante acceso por su API. De esta forma se facilita el uso en cualquier sistema operativo, aunque Amazon recomienda usar sus SDK (kits de desarrollo) de AWS [11] para comenzar a utilizar DynamoDB.

Este servcio de base de datos NoSQL completamente administrado, proporciona un rápido desempeño y escalabilidad. Una de las principales características es quitar la carga de trabajo derivada de la administración de la base de datos distribuida ya que aquí se encargaría Amazon Web Services, supliendo los problemas de aprovisionamiento, instalación, hardware, replicación y escalabilidad [10].

DynamoDB permite escalar fácilmente la base de datos, es decir, nos ayuda en administrar el software de la base de datos y el aprovisionamiento del hardware necesario, ya que con DynamoDB esto se realiza automáticamente en unos minutos. Además, automáticamente, realiza particiones constantes de los datos y aprovisiona capacidad adicional del servidor conforme crece el tamaño de la tabla. También, realiza una réplica sincronizada de los datos en tres centros de una misma región de Amazon Web Services para mantener siempre la disponibilidad y durabilidad de los datos.

DynamoDB puede ser configurada para que las lecturas devuelva un resultado que refleje todas las escrituras que se han respondido satisfactoriamente antes de realizar la consulta o como viene por defecto, que devuelva las respuestas menos actualizadas (por segundos), de esta forma se aumenta la velocidad de respuesta.

Amazon DynamoDB se ejecuta exclusivamente en SSD para mayor eficacia a cambio de aumentar el coste final al usuario. Por eso mismo el precio aumenta respecto a una base de datos implementada en HDD, por GB almacenado. Sin embargo suplen este precio cobrando por la carga que se le solicita, ya que los SSD al final “trabajan” menos porque realizan las peticiones de E/S antes .De esta forma en las cargas de trabajo habituales de las bases de datos, la factura total del servicio DynamoDB basado en SSD suele ser más baja que el costo de utilizar una base de datos normal relacional o no relacional basada en soportes giratorios.

Por esto mismo, el servicio de DynamoDB no es recomendable si se va a almacenar gran cantidad de datos a los que no se suele tener acceso, porque estaría cobrando por GB almacenado, lo que el precio es bastante más alto, además, el precio reflejado es el precio por almacenar en las 3 bases de datos que se ha explicado antes (la tuya y las dos de soporte).

De esta forma DynamoDB es ideal cuando se necesita un desarrollo rápido escalable y predecible. Además, soporta la estructura de datos clave-valor y de documentos. Amazon DynamoDB almacena datos estructurados, indexados por clave principal [12].

Respecto a un modelo tradicional relacional, DynamoDB no permite consultas características de los modelos relacionales como uniones, o transacciones complejas (NoSQL). Si se requiere un motor de base de datos supla estas funciones Amazin tiene RDS y EC2.
